﻿//----------------------------------------------------------------------------
// Gordon Lee
//
// F# Engineering Test
// Written in Microsoft Visual Studio 2008
//
// Reverse Words
// Reverses the words in a sentence but does not reverse the letters
//----------------------------------------------------------------------------

#light

open System
open System.IO
open System.Windows.Forms
open System.Text

// Form
let form = new Form(Width = 400, Height = 300, Visible = true, Text = "Gordon Lee - F# - Reverse Words", Menu = new MainMenu())

// Menu
let mFile = form.Menu.MenuItems.Add("&File")
let mHelp = form.Menu.MenuItems.Add("&Help")
let miNew = new MenuItem("&New")
let miOpen = new MenuItem("&Open")
let miSave = new MenuItem("&Save As...")
let miExit  = new MenuItem("&Exit")
let miAbout = new MenuItem("&About")
let _ = mFile.MenuItems.Add(miNew)
let _ = mFile.MenuItems.Add(miOpen)
let _ = mFile.MenuItems.Add(miSave)
let _ = mFile.MenuItems.Add(miExit)
let _ = mHelp.MenuItems.Add(miAbout)

// ReadFile dialog
let ReadFile () =
    let d = new OpenFileDialog() 
    d.Filter <- "Text Files *.txt|*.txt|All files *.*|*.*";
    d.FilterIndex <- 2;
    if d.ShowDialog() = DialogResult.OK then
        let str  = new StreamReader(d.FileName)
        let text = str.ReadToEnd()
        str.Close()
        Some (d.FileName,text)
    else
        None
        
// Filename state
let mutable filename = "*.txt"
let SetFilename f  = filename <- f

// SaveFile dialog
let SaveFile file text =
    let d = new SaveFileDialog() 
    d.Filter <- "Text Files *.txt|*.txt|All files *.*|*.*";
    d.FilterIndex <- 2;
    d.FileName <- file;
    d.AddExtension <- false;  
    d.OverwritePrompt <- true;
    if d.ShowDialog() = DialogResult.OK then
        let sw = new StreamWriter(d.FileName)
        sw.Write(text:string);
        sw.Close();

// RichTextBox
let textBoxInput = new RichTextBox()
textBoxInput.Dock <- DockStyle.Top
textBoxInput.Text <- "Hello World"
form.Controls.Add(textBoxInput)

let textBoxOutput = new RichTextBox()
textBoxOutput.Dock <- DockStyle.Bottom 
textBoxOutput.Text <- ""
textBoxOutput.ReadOnly <- true
form.Controls.Add(textBoxOutput)

// Button
let buttonOutput = new Button()
buttonOutput.Text <- "Reverse Words"
buttonOutput.Dock <- DockStyle.Right
form.Controls.Add(buttonOutput)

// Reverse Words algorithm
// 1. Split input string using spaces and newlines as delimiters into a Sequence of strings
// 2. Reverse the Sequence
// 3. Fold the Sequence of strings into a single string with a space between words using String Builder
// 4. Return in string form
let reverseWords (input : string) =
    input.Split(' ', '\n')
    |> Seq.fold (fun acc elem -> elem::acc) []
    |> Seq.fold (fun (sb : StringBuilder) elem -> sb.AppendFormat(" {0}", elem)) (StringBuilder())
    |> string

// Operations
let opNew _ =
    textBoxInput.Text <- ""
    textBoxOutput.Text <- ""
let opOpen _ =
    match ReadFile () with
    | Some (file,text) -> textBoxInput.Text <- text; textBoxOutput.Text <- ""
    | None -> ()
let opSave _ = SaveFile filename textBoxInput.Text
let opExit _ = form.Close()
let opAbout _ = 
    MessageBox.Show("Gordon Lee\n\
\n\
F# Engineering Test\n\
Written in Microsoft Visual Studio 2008\n\
\n\
Reverse Words\n\
Reverses the words in a sentence but does not reverse the letters","About") |> ignore
let opOutput _ =
    textBoxOutput.Text <- reverseWords textBoxInput.Text
    
// Callbacks
miNew.Click.Add(opNew)
miOpen.Click.Add(opOpen)
miSave.Click.Add(opSave)
miExit.Click.Add(opExit)
miAbout.Click.Add(opAbout)
buttonOutput.Click.Add(opOutput)

#if COMPILED
[<STAThread>]    
do Application.Run(form) |> ignore
#endif
